#include "lepton.h"
#include "usb_device.h"

static uint8_t imageFrame[LEPTON_FRAME_SIZE];
static uint8_t image[(IMAGE_LINES*(LEPTON_FRAME_SIZE - 4)) + 2];

static uint8_t sentImages = 3;
static SPI_HandleTypeDef spi;
static uint16_t failedFrames = 0;

void LeptonInit(SPI_HandleTypeDef spiHandler)
{
    image[0] = 0;
    image[1] = 0;
    spi = spiHandler;
    InitModule();   
    Start();
}

static void InitModule()
{
    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_4, GPIO_PIN_SET);
    HAL_Delay(100);
    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_4, GPIO_PIN_RESET);
    HAL_Delay(100);
}

static void SendImage()
{
    while(CDC_Transmit_FS(&image[0], ((LEPTON_FRAME_SIZE - 4) * IMAGE_LINES)+2) != USBD_OK);
    HAL_GPIO_TogglePin(GPIOC, GPIO_PIN_13);
}

static void CheckFailedConnection()
{
    if (failedFrames > 5000)
    {
        InitModule();
        failedFrames = 0;
        HAL_GPIO_TogglePin(GPIOC, GPIO_PIN_13);
    }
}

static void DiscardFrame()
{
    failedFrames++;
    SkipSpiImageData();
}

static void ImageReceived(uint8_t frameNumber)
{
    if (sentImages == 3)
    {
        sentImages = 0;
        SendImage();
    }
    sentImages++;
}

static uint8_t IsImageComplete(uint8_t frameNumber)
{
    return frameNumber == (IMAGE_LINES - 1);
}

static void ReceiveImageData()
{
    uint8_t frameNumber = imageFrame[1];

    if (frameNumber >= IMAGE_LINES)
    {
        SkipSpiImageData();
        return;
    }
    
    failedFrames = 0;
    HAL_SPI_Receive(&spi, (uint8_t *)&image[2 + (frameNumber*(LEPTON_FRAME_SIZE - 4))], (LEPTON_FRAME_SIZE - 4), 5);
    if (IsImageComplete(frameNumber))
    {
        ImageReceived(frameNumber);
    }
}

void Start()
{
    while (1)
    {
        CheckFailedConnection();
        HAL_SPI_Receive(&spi, (uint8_t *)&imageFrame[0], 4, 5);

        if (((imageFrame[0] & 0xf) == 0x0f))
        {
            DiscardFrame();
            continue;
        }
        
        ReceiveImageData();
    }
}

static void SkipSpiImageData()
{
    HAL_SPI_Receive(&spi, (uint8_t *)&imageFrame, (LEPTON_FRAME_SIZE - 4), 5);
}